package dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus;

import com.mojang.datafixers.kinds.Applicative;
import dev.shadowsoffire.placebo.codec.PlaceboCodecs;
import net.minecraftforge.registries.ForgeRegistries;
import com.mojang.datafixers.kinds.App;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import com.google.common.base.Preconditions;
import dev.shadowsoffire.attributeslib.api.IFormattableAttribute;
import dev.shadowsoffire.attributeslib.AttributesLib;
import net.minecraft.network.chat.Component;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemItem;
import java.util.UUID;
import java.util.function.BiConsumer;
import net.minecraft.world.item.ItemStack;
import dev.shadowsoffire.apotheosis.Apotheosis;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemClass;
import dev.shadowsoffire.placebo.util.StepFunction;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import java.util.Map;
import net.minecraft.world.entity.ai.attributes.AttributeModifier;
import net.minecraft.world.entity.ai.attributes.Attribute;
import com.mojang.serialization.Codec;

public class AttributeBonus extends GemBonus
{
    public static Codec<AttributeBonus> CODEC;
    protected final Attribute attribute;
    protected final AttributeModifier.Operation operation;
    protected final Map<LootRarity, StepFunction> values;
    
    public AttributeBonus(final GemClass gemClass, final Attribute attr, final AttributeModifier.Operation op, final Map<LootRarity, StepFunction> values) {
        super(Apotheosis.loc("attribute"), gemClass);
        this.attribute = attr;
        this.operation = op;
        this.values = values;
    }
    
    @Override
    public void addModifiers(final ItemStack gem, final LootRarity rarity, final BiConsumer<Attribute, AttributeModifier> map) {
        map.accept(this.attribute, this.read(gem, rarity, GemItem.getUUIDs(gem).get(0)));
    }
    
    @Override
    public Component getSocketBonusTooltip(final ItemStack gem, final LootRarity rarity) {
        return (Component)IFormattableAttribute.toComponent(this.attribute, this.read(gem, rarity, UUID.randomUUID()), AttributesLib.getTooltipFlag());
    }
    
    @Override
    public AttributeBonus validate() {
        Preconditions.checkNotNull((Object)this.attribute, (Object)"Invalid AttributeBonus with null attribute");
        Preconditions.checkNotNull((Object)this.operation, (Object)"Invalid AttributeBonus with null operation");
        Preconditions.checkNotNull((Object)this.values, (Object)"Invalid AttributeBonus with null values");
        return this;
    }
    
    @Override
    public boolean supports(final LootRarity rarity) {
        return this.values.containsKey(rarity);
    }
    
    @Override
    public int getNumberOfUUIDs() {
        return 1;
    }
    
    public AttributeModifier read(final ItemStack gem, final LootRarity rarity, final UUID id) {
        return new AttributeModifier(id, "apoth.gem_modifier", (double)this.values.get(rarity).get(0.0f), this.operation);
    }
    
    public Codec<? extends GemBonus> getCodec() {
        return AttributeBonus.CODEC;
    }
    
    static {
        AttributeBonus.CODEC = (Codec<AttributeBonus>)RecordCodecBuilder.create(inst -> inst.group((App)GemBonus.gemClass(), (App)ForgeRegistries.ATTRIBUTES.getCodec().fieldOf("attribute").forGetter(a -> a.attribute), (App)PlaceboCodecs.enumCodec((Class)AttributeModifier.Operation.class).fieldOf("operation").forGetter(a -> a.operation), (App)AttributeBonus.VALUES_CODEC.fieldOf("values").forGetter(a -> a.values)).apply((Applicative)inst, AttributeBonus::new));
    }
}
